#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <string.h>
#include <math.h>

#include "ws.h"
#include <common/ozrp_ws.h>

#define bzero(s, c) memset(s, 0, c)

wonder_swan::wonder_swan(const std::string devfile)
  : sio_name(devfile)
{
}

wonder_swan::~wonder_swan()
{
}

void
wonder_swan::init(void)
{
  this->sio = open(sio_name.c_str(), O_RDWR);
  if(this->sio == -1){
    perror("open()");
    exit(1);
  }
  {
    struct termios tio;  

    bzero((char *)&tio, sizeof(tio));

    tio.c_cflag = CS8 | CREAD;
    cfsetospeed(&tio, B38400);
    cfsetispeed(&tio, B38400);

    tcsetattr(this->sio, TCSANOW, &tio);
  }
}

void
wonder_swan::send_show_data(const ozrp_show_data_t& show)
{
  unsigned char buf[512];
  int i = 0;
  // TIME
  buf[i++] = (show.time >> 7) & 0x7f;
  buf[i++] = (show.time & 0x7f);

  // BALL
  buf[i++] = (unsigned char) rint((show.ball.x + 57.5)
				  * WS_LENGTH_MAGNIFY);
  buf[i++] = (unsigned char) rint((show.ball.y + 39)
				  * WS_LENGTH_MAGNIFY);

  // PLAYER
  int j;
  for (j = 0; j < 22; j++) {
    buf[i++] = show.players[j].type;
    buf[i++] = (unsigned char) rint((show.players[j].x + 57.5) 
				    * WS_LENGTH_MAGNIFY);
    buf[i++] = (unsigned char) rint((show.players[j].y + 39)
				    * WS_LENGTH_MAGNIFY);
  }

  // HEADER
  {
    char head_buf[8];
    head_buf[0] = 0xff;
    head_buf[1] = PKT_PW_SHOW_DATA;
    head_buf[2] = (i >> 7) & 0x7f;
    head_buf[3] = (i & 0x7f);

    // SEND
    write(this->sio, head_buf, 4);
    write(this->sio, buf, i);
  }
}

void
wonder_swan::send_game_data(const ozrp_game_data_t& game)
{
  unsigned char buf[32];
  int i = 0;

  // TIME
  buf[i++] = (game.time >> 7) & 0x7f;
  buf[i++] = (game.time & 0x7f);

  buf[i++] = (unsigned char) game.play_mode;
  buf[i++] = (unsigned char) game.my_score;
  buf[i++] = (unsigned char) game.their_score;

  // HEADER
  {
    char head_buf[8];
    head_buf[0] = 0xff;
    head_buf[1] = PKT_PW_GAME_DATA;
    head_buf[2] = (i >> 7) & 0x7f;
    head_buf[3] = (i & 0x7f);

    // SEND
    write(this->sio, head_buf, 4);
    write(this->sio, buf, i);
  }
}

void
wonder_swan::send_body_data(const ozrp_body_data_t& body)
{
  unsigned char buf[32];
  int i = 0;
  int tmp = 0;
  // TIME
  buf[i++] = (body.time >> 7) & 0x7f;
  buf[i++] = (body.time & 0x7f);

  tmp = (int) rint(body.stamina);
  buf[i++] = (tmp >> 7) & 0x7f;
  buf[i++] = (tmp & 0x7f);

  tmp = (int) rint(body.stamina / 4000 * 100); // STAMINA MAX 4000
  buf[i++] = (tmp >> 7) & 0x7f;
  buf[i++] = (tmp & 0x7f);

  tmp = (int) rint(body.effort);
  buf[i++] = (tmp >> 7) & 0x7f;
  buf[i++] = (tmp & 0x7f);

  tmp = (int) rint(body.recovery);
  buf[i++] = (tmp >> 7) & 0x7f;
  buf[i++] = (tmp & 0x7f);


  // HEADER
  {
    char head_buf[8];
    head_buf[0] = 0xff;
    head_buf[1] = PKT_PW_BODY_DATA;
    head_buf[2] = (i >> 7) & 0x7f;
    head_buf[3] = (i & 0x7f);

    // SEND
    write(this->sio, head_buf, 4);
    write(this->sio, buf, i);
  }
}

void
wonder_swan::send_pointer_command(const int& onoff)
{
  char buf[3];
  buf[0] = 0xff;
  buf[1] = PKT_PW_POINTER_CMD;
  buf[2] = onoff;
  write(this->sio, buf, 3);
}

void
wonder_swan::send_exit_command()
{
  char buf[2];
  buf[0] = 0xff;
  buf[1] = PKT_PW_EXIT_CMD;
  write(this->sio, buf, 2);
}


int 
wonder_swan::recv_data()
{
  unsigned char buf[4];
  int n = read(this->sio, buf, 4);
  int i = 0;

  for (i = 0; i < n; i++) {
    ::printf(" %02x", (unsigned char) buf[i]);
  }
  ::printf("\n");

  i = 0;

  while (true) {
    while (buf[i++] != (unsigned char) 0x0ff) {
      ;
    }

    switch (buf[i++]) {
    case PKT_WP_KEYSTATUS:
      {
	int h = buf[i++];
	int l = buf[i++];
	this->key = (((h << 8) | l) & 0x0ffe);
      }
      break;
    case PKT_WP_POINTER:
      {
	int xx = buf[i++];
	int yy = buf[i++];
	this->x = (((double) xx) / WS_LENGTH_MAGNIFY) - 57.5;
	this->y = (((double) yy) / WS_LENGTH_MAGNIFY) - 39;
      }
      break;
    case PKT_WP_ACK:
      break;
    }
    if (i >= n) {
      break;
    }
  }
  return n;
}

