#include <cstdio>
#include <cmath>
extern "C" {
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
}

#include <ws.h>
#include <common/ozrp_ws.h>
#include <ozrp.h>

#include "udp_socket.h"
#include "player_command.h"


void ws_pad(player_command& pc, wonder_swan& pad, double body_angle)
{
  if (pad.key & KEY_X4) {
    pc.kick(100, 0);
    return;
  }else if (pad.key & KEY_X3) {
    pc.kick(40, 0);
    return;
  }else if (pad.key & KEY_X2) {
    pc.kick(20, 0);
    return;
  }
  int x = 0;
  int y = 0;
  if (pad.key & KEY_Y1) {    y--;  }
  if (pad.key & KEY_Y2) {    x++;  }
  if (pad.key & KEY_Y3) {    y++;  }
  if (pad.key & KEY_Y4) {    x--;  }

  if (x == 0 && y == 0) {
    return;
  } else {
    double angle = atan2(y, x) * 180 / M_PI;
    double diff = angle - body_angle;
    while (diff > 180){
      diff -= 360;
    }
    while (diff < -180){
      diff += 360;
    }

    printf("%f %f\n", angle, diff);
    if (fabs(diff) > 20) {
      if (fabs(diff) > 90) {
	diff /= 2;
      }
      pc.turn(diff);
      return;
    }
  }
  {
    pc.dash(100);
  }
}

ozrp_show_data_t show_data;

int
fullstate_info(std::string& str, player_command& pc,
	       wonder_swan& pad, double& body_angle)
{
  if (::sscanf(str.c_str(), "(fullstate %d", &(show_data.time)) == 1) {
    char *p = ::strstr(str.c_str(), "(ball" );
    if (p == NULL) {
      return 0;
    } else {
      char side;
      int unum;
      int i;
      double vx, vy, neck_ang, stamina, effort, recovery;
      int n = ::sscanf(p, "(ball %lf %lf %lf %lf)",
		       &(show_data.ball.x), &(show_data.ball.y),
		       &vx, &vy);
      if (n != 4) {
	return 0;
      }
      
      for (i = 0; i < 22; i++) {
	p++;
	p = ::index(p, '(');
	if (p == NULL) {
	  break;
	}
	//::printf("%s\n", p);
	int n = ::sscanf(p, "(%c_%d %lf %lf %lf %lf %lf %lf %lf %lf %lf)",
			 &side, &unum,
			 &(show_data.players[i].x),
			 &(show_data.players[i].y),
			 &vx, &vy,
			 &(show_data.players[i].angle),
			 &neck_ang, &stamina, &effort, &recovery);
	if (n == 11) {
	  if (pc.side == side) {
	    if (pc.number == unum) {
	      show_data.players[i].type = PT_SELF;
	      body_angle = show_data.players[i].angle;
	    } else {
	      show_data.players[i].type = PT_MEMBER;
	    }
	  } else {
	    show_data.players[i].type = PT_ENEMY;
	  }
	} else {
	  return 0;
	}
      }
      for (; i < 22; i++) {
	show_data.players[i].type = PT_UNSEEN;
      }
      pad.send_show_data(show_data);
      //::printf("send data.\n");
      return 1;
    }
  }
  return 0;
}


void message_loop(player_command& pc, wonder_swan& pad)
{
  udp_socket&	sock=pc.socket();
  fd_set	readfds;
  int		max_fd, n, ret ;

  while(1){
    timeval timeout;
    timeout.tv_sec = 0;
    timeout.tv_usec = 1000;

    FD_ZERO(&readfds) ;

    FD_SET(sock.fd(), &readfds) ;
    max_fd = sock.fd();

    FD_SET(pad.fd(), &readfds) ;
    max_fd = (max_fd < pad.fd())? pad.fd(): max_fd;
    
    if((ret = ::select(max_fd + 1, &readfds, NULL, NULL, &timeout)) < 0){
      perror("select") ;
      break ;
    } else if (ret != 0) {
      string str;
      if(FD_ISSET(sock.fd(), &readfds)){
	n = sock.recv(str) ;
	if(n == -1)
	  break ;
	else if(n > 0){
	  double body_angle;
	  int res = fullstate_info(str, pc, pad, body_angle);
	  if (res == 1) {
	    ws_pad(pc, pad, body_angle);
	  }
	}
	fflush(stdout) ;
      }else if (FD_ISSET(pad.fd(), &readfds)){
	pad.recv_data();
	if (pad.key & KEY_A && pad.key & KEY_START) {
	  return;
	}
      } 
    }
  }
}

int 
wait_by_select(wonder_swan& ws)
{
  fd_set fds;
  timeval timeout;

  FD_ZERO(&fds);
  FD_SET(ws.fd(), &fds);
  timeout.tv_sec = 0;
  timeout.tv_usec = 1000;
    
  int res = ::select(ws.fd() + 1, &fds, NULL, NULL, &timeout);
  if (res > 0) {
    ws.recv_data();
  } else if (res < 0) {
    perror("select()");
  }
  return res;
}


int main(int argc, char **argv)
{
  player_command pc;
  wonder_swan    pad;

  if(pc.socket().fd() == -1) {
    exit(1) ;
  }
  pc.init("xxx");
  pad.init();

  while (::wait_by_select(pad) <= 0) {
    ;
  }
  cout << "connect" << endl;

  message_loop(pc, pad) ;
  pc.bye();
  pad.send_exit_command();
  pad.send_exit_command();
}


