#include <ozrp_ws.h>
/*
#include "rclogo1.h"
#include "rclogo3.h"
*/
#include "rclogo2.h"

unsigned short field[18][26][8];
unsigned short mode_line1[18][8];
unsigned short mode_line2[18][8];

unsigned int field_top;
unsigned int mode_line1_top;
unsigned int mode_line2_top;

unsigned int player_fn;
unsigned int pointer_fn;
unsigned int ball_fn;

void 
ozrp_pset(int x, int y, int col) {
  if (y / 8 < 18 && x / 8 < 26) {
    short bits;
    switch (col) {
    default:
    case 0: bits = 0x0000; break;
    case 1: bits = 0x0001; break;
    case 2: bits = 0x0100; break;
    case 3: bits = 0x0101; break;
    };
    field[y / 8][x / 8][y % 8] &= (0xffff - (0x0101 << (7- (x % 8)))); 
    field[y / 8][x / 8][y % 8] |= (bits << (7 - (x % 8))); 
  }
}

void
ozrp_hline(int x0, int y, int x1, int col)
{
  int start = (x0 < x1)? x0: x1;
  int end =   (x0 < x1)? x1: x0;

  for (;start <= end; start++) {
    ozrp_pset(start, y, col);
  }
}

void
ozrp_vline(int x, int y0, int y1, int col)
{
  int start = (y0 < y1)? y0: y1;
  int end =   (y0 < y1)? y1: y0;

  for (;start <= end; start++) {
    ozrp_pset(x, start, col);
  }
}

void
ozrp_box(int x, int y, int x2, int y2, int col)
{
  ozrp_hline(x,  y,  x2, col);
  ozrp_hline(x,  y2, x2, col);
  ozrp_vline(x,  y,  y2, col);
  ozrp_vline(x2, y,  y2, col);
}

void
ozrp_fill_box(int x, int y, int x2, int y2, int col)
{
  for (;y < y2; y++) {
    ozrp_hline(x,  y,  x2, col);
  }
}

void
ozrp_circle(int x, int y, int r, int col)
{
  int a , b , u , v , uu , vv , r2 ;

  r2 = r * r ;

  a  = r     ;
  b  = 0     ;
  u  = r2-r  ;
  v  = 0     ;
  uu = r*2-2 ;
  vv = 1     ;

  while ( a >= b ) {
    ozrp_pset( x + a , y + b , col ) ;
    ozrp_pset( x + a , y - b , col ) ;
    ozrp_pset( x - a , y + b , col ) ;
    ozrp_pset( x - a , y - b , col ) ;
    ozrp_pset( x + b , y + a , col ) ;
    ozrp_pset( x + b , y - a , col ) ;
    ozrp_pset( x - b , y + a , col ) ;
    ozrp_pset( x - b , y - a , col ) ;
    b++ ;
    v += vv , vv += 2 ;
    if ( r2 - v < u ) {
      --a ;
      u -= uu , uu -= 2 ;
    }
  }
}


void draw_field();

unsigned short player[] = {
  0x0000,  /* 0000 0000 */
  0x0000,  /* 0000 0000 */
  0x1800,  /* 0002 2000 */
  0x3C18,  /* 0023 3200 */
  0x3C18,  /* 0023 3200 */
  0x1800,  /* 0002 2000 */
  0x0000,  /* 0000 0000 */
  0x0000,  /* 0000 0000 */
};

unsigned short ball[] = {
  0x0000,  /* 0000 0000 */
  0x0000,  /* 0000 0000 */
  0x1800,  /* 0003 3000 */
  0x3C00,  /* 0033 3300 */
  0x3C00,  /* 0033 3300 */
  0x1800,  /* 0003 3000 */
  0x0000,  /* 0000 0000 */
  0x0000,  /* 0000 0000 */
#if 0
  0x0000,  /* 0000 0000 */
  0x1818,  /* 0003 3000 */
  0x3c24,  /* 0032 2300 */
  0x7E7E,  /* 0333 3330 */
  0x7e5A,  /* 0323 3230 */
  0x3c24,  /* 0032 2300 */
  0x1818,  /* 0003 3000 */
  0x0000,  /* 0000 0000 */
#endif
};

unsigned short pointer[] = {
  0x00C3,
  0x00E7,
  0x007E,
  0x0024,
  0x0024,
  0x007E,
  0x00E7,
  0x00C3, 
};


void
ozrp_display_init()
{
  memset(field, 0xff, 26*18*16) ;
  memset(mode_line1, 0, 18*16 ) ;
  memset(mode_line2, 0, 18*16 ) ;

  display_control(DCM_SCR1 | DCM_SPR);
  wwc_set_color_mode(COLOR_MODE_4COLOR);
  sprite_set_range(0, 24);
  
  {
    static unsigned short field_cmap[26*18];
    static unsigned short mode_line_cmap[18];
    int i, j, k, c;

    wwc_palette_set_color(PLT_FIELD, 3, 0x0f0);
    wwc_palette_set_color(PLT_FIELD, 2, 0x000);
    wwc_palette_set_color(PLT_FIELD, 1, 0x5f5);
    wwc_palette_set_color(PLT_FIELD, 0, 0xfff);

    wwc_palette_set_color(PLT_SELF,    3, 0x00f);
    wwc_palette_set_color(PLT_SELF,    2, 0xfff);
    wwc_palette_set_color(PLT_SELF,    1, 0xf00);
    wwc_palette_set_color(PLT_MEMBER,  3, 0xff0);
    wwc_palette_set_color(PLT_ENEMY,   3, 0xf00);
    wwc_palette_set_color(PLT_UNKNOWN, 3, 0x000);
    wwc_palette_set_color(PLT_MEMBER,  2, 0xff0);
    wwc_palette_set_color(PLT_ENEMY,   2, 0xf00);
    wwc_palette_set_color(PLT_UNKNOWN, 2, 0x000);

    {
      int l = 0;
      for (j = 0; j < 12; j++) {
	for (i = 0; i < 7; i++) {
	  for (k = 0; k < 8; k++, l++) {
	    field[3+j][10+i][k] = bmp_rclogo2[l];
	  }
	}
      }
    }

    draw_field();

    field_top = c = 0;
    font_set_colordata(field_top, 26*18, field);
    for (i = 0; i < 26*18; i++, c++) {
      field_cmap[i] = (PLT_FIELD << 9) |  c;
    }
    screen_set_char(SCREEN1, 2, 0, 26, 18, field_cmap);

    mode_line1_top = c;
    font_set_colordata(mode_line1_top, 18, mode_line1);
    for (i = 0; i < 18; i++, c++) {
      mode_line_cmap[i] = c;
    }
    screen_set_char(SCREEN1, 1, 0, 1, 18, mode_line_cmap);

    mode_line2_top = c;
    font_set_colordata(mode_line2_top, 18, mode_line2);
    for (i = 0; i < 18; i++, c++) {
      mode_line_cmap[i] = c;
    }
    screen_set_char(SCREEN1, 0, 0, 1, 18, mode_line_cmap);

    player_fn = c++;
    font_set_colordata(player_fn, 1, player);
    ball_fn = c++;
    font_set_colordata(ball_fn, 1, ball);
    pointer_fn = c++;
    font_set_colordata(pointer_fn, 1, pointer);

    sprite_set_char(0, CFM_SPR_UPPER | (PLT_BALL - 8) * 0x200 | ball_fn);
    sprite_set_char(23, 
		    CFM_SPR_UPPER | (PLT_POINTER - 8) * 0x200 | pointer_fn);
    for (i = 0; i < 24; i++) {
      sprite_set_location(i, 0, 200);
    }
    /*
    sprite_set_location(0, 32, 32);
    sprite_set_char(1, CFM_SPR_UPPER | (PLT_SELF - 8) * 0x200 | player_fn);
    sprite_set_location(1, 42, 32);
    sprite_set_char(2, CFM_SPR_UPPER | (PLT_MEMBER - 8) * 0x200 | player_fn);
    sprite_set_location(2, 52, 32);
    sprite_set_char(3, CFM_SPR_UPPER | (PLT_ENEMY - 8) * 0x200 | player_fn);
    sprite_set_location(3, 62, 32);
    sprite_set_char(4, CFM_SPR_UPPER | (PLT_UNKNOWN - 8) * 0x200 | player_fn);
    sprite_set_location(4, 72, 32);
    */
  }
}

#if 0
hline_end(struct ws_point end, int col)
{
  int i;
  short bits;
  short mask;
  switch (col) {
  default:
  case 0: bits = 0x0000; break;
  case 1: bits = 0x0001; break;
  case 2: bits = 0x0100; break;
  case 3: bits = 0x0101; break;
  };

  for (i = 0, mask = 0x0000; i < end.bit.x; i++) {
    mask |= (0x0101 << i);
  }
  field[end.ch.y][end.ch.x][end.bit.y] &= mask; 

  for (i = end.bit.x; i < 8; i++) {
    field[end.ch.y][end.ch.x][end.bit.y] |= bits << i;
  }
}

static void
hline_start(struct ws_point start, int col)
{
  int i;
  short bits;
  short mask;
  switch (col) {
  default:
  case 0: bits = 0x0000; break;
  case 1: bits = 0x0010; break;
  case 2: bits = 0x1000; break;
  case 3: bits = 0x1010; break;
  };

  for (i = 0, mask = 0x0000; i < start.bit.x; i++) {
    mask |= (0x1010 >> i);
  }
  field[start.ch.y][start.ch.x][start.bit.y] &= mask; 

  for (i = start.bit.x; i < 8; i++) {
    field[start.ch.y][start.ch.x][start.bit.y] |= bits >> i;
  }
}


void
ozrp_hline(int x0, int y, int x1, int col)
{
  static struct ws_point start;
  static struct ws_point end;
  int start_x = (x0 < x1)? x0: x1;
  int end_x =   (x0 < x1)? x1: x0;

  start.ch.y  = end.ch.y  = y / 8;
  start.bit.y = end.bit.y = y % 8;
  
  start.ch.x  = start_x / 8;
  start.bit.x = start_x % 8;

  end.ch.x  = end_x / 8;
  end.bit.x = end_x % 8;

  {
    int i;
    short bits;
    switch (col) {
    default:
    case 0: bits = 0x0000; break;
    case 1: bits = 0x00ff; break;
    case 2: bits = 0xff00; break;
    case 3: bits = 0xffff; break;
    };
    hline_start(start, col);
    for (i = start.ch.x + 1; i < end.ch.x; i++) {
      field[start.ch.y][i][start.bit.y] = bits;
    }
    hline_end(end, col);
  }
}
#endif
